From 5947df9c94655116c8905b2540d039eb0d18ce5f Mon Sep 17 00:00:00 2001
From: Vignesh Raghavendra <vigneshr@ti.com>
Date: Fri, 11 Jun 2021 18:29:31 +0530
Subject: [PATCH] ARM: mach-k3 sysfw-loader: Copy sysfw.itb to OCRAM in
 OSPI/SPI bootmode

In case of xSPI bootmode OSPI flash needs to be accessed using 16bit
accessors. Hence we cannot parse sysfw.itb FIT image directly on OSPI
flash via MMIO window. So, copy the image to internal on-chip RAM before
parsing the image.

Signed-off-by: Vignesh Raghavendra <vigneshr@ti.com>
---
 arch/arm/mach-k3/sysfw-loader.c | 16 ++++++++++++++--
 1 file changed, 14 insertions(+), 2 deletions(-)

diff --git a/arch/arm/mach-k3/sysfw-loader.c b/arch/arm/mach-k3/sysfw-loader.c
index 2a75ac1ee3..40aa9c334c 100644
--- a/arch/arm/mach-k3/sysfw-loader.c
+++ b/arch/arm/mach-k3/sysfw-loader.c
@@ -21,6 +21,7 @@
 #include <dm/uclass-internal.h>
 #include <spi_flash.h>
 
+#include <asm/io.h>
 #include <asm/arch/sys_proto.h>
 #include "common.h"
 
@@ -338,6 +339,14 @@ static void *k3_sysfw_get_spi_addr(void)
 
 	return (void *)(addr + CONFIG_K3_SYSFW_IMAGE_SPI_OFFS);
 }
+
+static void k3_sysfw_spi_copy(u32 *dst, u32 *src, size_t len)
+{
+	size_t i;
+
+	for (i = 0; i < len / sizeof(*dst); i++)
+		*dst++ = *src++;
+}
 #endif
 
 void k3_sysfw_loader(bool rom_loaded_sysfw,
@@ -347,6 +356,7 @@ void k3_sysfw_loader(bool rom_loaded_sysfw,
 	struct spl_image_info spl_image = { 0 };
 	struct spl_boot_device bootdev = { 0 };
 	struct ti_sci_handle *ti_sci;
+	void *sysfw_spi_base;
 	int ret = 0;
 
 	if (rom_loaded_sysfw) {
@@ -397,9 +407,11 @@ void k3_sysfw_loader(bool rom_loaded_sysfw,
 #endif
 #if CONFIG_IS_ENABLED(SPI_LOAD)
 	case BOOT_DEVICE_SPI:
-		sysfw_load_address = k3_sysfw_get_spi_addr();
-		if (!sysfw_load_address)
+		sysfw_spi_base = k3_sysfw_get_spi_addr();
+		if (!sysfw_spi_base)
 			ret = -ENODEV;
+		k3_sysfw_spi_copy(sysfw_load_address, sysfw_spi_base,
+				  CONFIG_K3_SYSFW_IMAGE_SIZE_MAX);
 		break;
 #endif
 #if CONFIG_IS_ENABLED(YMODEM_SUPPORT)
-- 
2.32.0

